/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 * 28-Aug-2012     Oleg Pevzner       7984 Java upgrade from 6 to 7
 */

package com.nemmco.participantbatcher.util;

import java.io.BufferedInputStream;

import java.net.URL;
import java.net.URLConnection;

import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;

import java.util.Enumeration;
import java.util.Properties;

import javax.crypto.Cipher;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESKeySpec;

import org.apache.commons.codec.DecoderException;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.binary.Hex;
import org.apache.log4j.Logger;

/**
 * This class is used to encrypt and decrypt passwords.
 * It has a main() routine to do the encrypt
 * It has a decrypt that is called in PropertiesReader to convert the password
 * <p>
 * @author Phil Hayes
 * version 1.0 - 18/05/2004
 * version 1.3 - 19-May-2004 converted to use in ParticipantBatcher
 * version 1.4.4 6-May-2006 Fixed javaDoc and coding standards
 * version 2.0.0 - 08-Apr-2010   Bob Jones   6356 Rewritten to use standard Java encryption
 *                           following the death of Cryptix.
 * @version 2.0.2 - 19-Nov-2010 Peer review work
 */

public class Crypt {

   //private static Crypt secObject;
   private static Logger log = Logger.getLogger(Crypt.class);

   private static final String CIPHER = "DES/ECB/NoPadding";
   private static final String ALGO = "DES";
   private static Cipher desCipher;

   /** Creates new Security object
    */
   private Crypt() {}

   /**
    * get the Secret Key from pb_key.properties file
    */
   public static SecretKey getKey() {
      // extract encryption key
      String theSeed = readHash("pb_key");
      byte[] hexSeed = {};
      DESKeySpec ks = null;
      SecretKeyFactory kf = null;
      SecretKey key = null;
      try {
         Hex hex = new Hex();
         try {
            hexSeed = (byte[])hex.decode(theSeed);
         } catch (DecoderException e) {
            log.error("Error decoding key seed to Hex byte array: ", e);
         }
         ks = new DESKeySpec(hexSeed);
      } catch (InvalidKeyException e) {
         log.error("Error getting key spec: ", e);
      }
      try {
         kf = SecretKeyFactory.getInstance(ALGO);
      } catch (NoSuchAlgorithmException e) {
         log.error("Error getting key factory: ", e);
      }
      try {
         key = kf.generateSecret(ks);
      } catch (InvalidKeySpecException e) {
         log.error("Error generating key: ", e);
      }
      return key;
   }

   /**
    * This class is used to set the only copy of this object.
    */
   //private static class SingletonCryptHolder {
   //private static Crypt instance = new Crypt();
   //}

   /**
    * Gets the cipher
    *
    * @return cipher
    * @throws NoSuchAlgorithmException
    * @throws NoSuchPaddingException
    */
   public static Cipher getCipher() throws NoSuchAlgorithmException, NoSuchPaddingException {
      if (desCipher == null) {
         desCipher = Cipher.getInstance(CIPHER);
      }

      return desCipher;
   }

   /**
    * encrypt the string and return encrypted.
    *
    * @param toEncrypt the string to encrypt
    * @return the string encrypted
    */
   public static String encrypt(SecretKey key, String toEncrypt) throws ApplicationException {

      // Ensure security provider is registered - thread safe
      //secObject = SingletonCryptHolder.instance;

      try {
         desCipher = getCipher();
         desCipher.init(Cipher.ENCRYPT_MODE, key);
      } catch (Exception e) {
         log.error("Error processing encrypt: ", e);
         return null;
      }

      return encrypt(desCipher, toEncrypt);
   }

   /**
    * This method encrypts the given parameter with the cipher.
    * This method will return an encrypted string or null when error occured.
    *
    * @param encryptCipher
    * @param toEncrypt
    * @return encrypted text
    */
   public static String encrypt(Cipher encryptCipher, String toEncrypt) {
      String encryptedText = null;

      if (toEncrypt != null) {
         byte[] workText = null;
         byte[] clearText = null;
         byte[] cipherText = null;
         int lenPassword = 0;
         int divWork8 = 0;
         int modWork8 = 0;
         int lenToEncrypt = 0;
         workText = toEncrypt.getBytes();
         lenPassword = workText.length;
         divWork8 = lenPassword / 8;
         modWork8 = lenPassword % 8;
         lenToEncrypt = divWork8 * 8;
         // password must be padded out to 8 byte multiples with 0x00
         if (modWork8 != 0) {
            lenToEncrypt += 8;
         }
         clearText = new byte[lenToEncrypt];
         System.arraycopy(workText, 0, clearText, 0, lenPassword);

         // encrypt the cleartext
         try {
            cipherText = encryptCipher.doFinal(clearText);
            encryptedText = new String(new Base64().encode(cipherText));
         } catch (Exception e) {
            ;
         }
      }
      return encryptedText;
   }

   /**
    * Public accessor method to decrypt the encrypted string.
    * It returns the decrypted string or null if error occured.
    *
    * @param toDecrypt
    * @return
    */
   public static String decrypt(String toDecrypt) {
      String hasbeenDecrypted = null;
      SecretKey key = getKey();
      hasbeenDecrypted = Crypt.decrypt(key, toDecrypt);
      return hasbeenDecrypted;
   }

   /**
    * This method decrypts the encrypted string with the given key.
    * It returns the decrypted string or null if error occured.
    *
    * @param key
    * @param toDecrypt
    * @return
    */
   public static String decrypt(SecretKey key, String toDecrypt) {

      // Ensure security provider is registered - thread safe
      //secObject = SingletonCryptHolder.instance;

      try {
         desCipher = getCipher();
         desCipher.init(Cipher.DECRYPT_MODE, key);
      } catch (Exception e) {
         log.error("Error processing decrypt: ", e);
         return null;
      }

      return decrypt(desCipher, toDecrypt);
   }

   /**
    * This method decrypts the encrypted string with the given cipher.
    * It returns the decrypted string or null if error occured.
    *
    * @param decryptCipher
    * @param toDecrypt
    * @return
    */
   public static String decrypt(Cipher decryptCipher, String toDecrypt) {
      byte[] ciphertext = null;
      byte[] cleartext = null;

      String decryptedText = null;

      if (toDecrypt != null) {
         System.out.println("toDecrypt = " + toDecrypt);
         ciphertext = toDecrypt.getBytes();

         // encrypt the cleartext
         try {
            cleartext = decryptCipher.doFinal(new Base64().decode(ciphertext));

            // cleartext will be padded out to 8 byte multiples with 0x00
            // needed for backward compatibility with operational method used when we used Cryptix
            // which used the OneAndZeroes padding method outlined above.
            // This padding needs to be removed before password can be used.
            int posPadding = cleartext.length;
            for (int i = 0; i < cleartext.length; ++i) {
               // find start 0x00 which is 0
               if (cleartext[i] == 0) {
                  posPadding = i;
                  break;
               }
            }
            decryptedText = new String(cleartext, 0, posPadding);
         } catch (javax.crypto.IllegalBlockSizeException e) {
            log.debug(""); // one-way display only wanted
         } catch (javax.crypto.BadPaddingException e) {
            log.debug("");
         }
      }
      return decryptedText;
   }

   /** The main method.
    * This is the entry point of this auxilliary application for testing.
    * <p>
    * @param args password
    */
   public static void main(String[] args) throws ApplicationException {

      if (args.length != 1) {

         System.out.println("Usage: java -classpath \"......\" " + "com.nemmco.participantbatcher.util.Crypt " + "password");
      } else {

         System.out.println("Encrypting - " + args[0]);
         SecretKey key = getKey();
         String encrypted = Crypt.encrypt(key, args[0]);
         System.out.println("Returns - " + encrypted);
         System.out.println("Decrypting - " + Crypt.decrypt(encrypted));
      }

   }

   /** this method opens the pb_key.properties file and returns the hash key
    *
    * @param name name of the name.properties file
    * @return the hask key
    *
    */
   public static String readHash(String name) {

      // An empty string on return indicates an error
      String hashString = "";
      try {

         // First, load the file into a Properties object
         URL url = null;
         String propertiesFileName = name + ".properties";
         url = PropertiesReader.class.getClassLoader().getResource(propertiesFileName);

         // if file reference exists
         if (url != null) {

            // open properties file
            URLConnection conn = url.openConnection();
            conn.connect();

            BufferedInputStream data = new BufferedInputStream(conn.getInputStream());
            Properties properties = new Properties();
            try {
               properties.load(data);
            } finally {
               data.close();
            }

            String pName = "";
            String pValue = "";

            // Now, let go through all properties, but just want hash=
            Enumeration<?> names = properties.propertyNames();
            while (names.hasMoreElements()) {

               // find name and value and store for return
               pName = (String)names.nextElement();
               pValue = properties.getProperty(pName);
               if ("hash".equalsIgnoreCase(pName)) {
                  hashString = pValue;
               }
            }
         }
      } catch (Exception e) {
         log.error("Exception getting hashString: ", e);
      }

      return hashString;
   }
}
