/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 * 28-Aug-2012     Oleg Pevzner       7984 Java upgrade from 6 to 7
 */

package com.nemmco.participantbatcher.util;

import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketTimeoutException;

import org.apache.log4j.Logger;

/**
 * Provides a web page to view the progress of the batcher.
 * When a user tries to use a web page the port request
 * to a port is responded to by calling HttpClient to
 * interact with the user.
 *
 * @author Mike Muir
 * version 1.4.4 6-May-2006 Fixed javaDoc and coding standards, made one thread,
 * @version 2.0.1 - 15-Nov-2010 Peer review work
 */
public class HttpServer extends Thread {

   private int port = 0;
   private volatile boolean stop = false;
   private static Logger logger = Logger.getLogger(HttpServer.class);

   /** Creates a new instance of HttpServer.
    *
    * get the port from constants to expose for HTTP requests.
    */
   public HttpServer() {
      this.port = Constants.getConstant().getPort();
   }

   /** do the wait on the port as a thread to avoid blocks.
    */
   public void run() {

      // only run if port number is specified
      if (port > 0) {

         try {

            // make a client to give the answer
            HttpClient hc = new HttpClient();

            // create server socket and wait for 31 sec
            ServerSocket ss = new ServerSocket(port);
            ss.setSoTimeout(31000);

            // if the client is running it calls for refresh every 30 sec
            while (!stop) {

               try {

                  // wait on server socket
                  Socket client = ss.accept();

                  // respond to socket by calling HttpClient routine
                  hc.screenRefresh(client);

                  // timeout loop so that a stop request is honoured
               } catch (SocketTimeoutException e) {
                  ;
               }

            }

         } catch (Exception e) {
            Util_Logger.warning(logger, "HttpServerWait calling ClientThread exception", e);
         }
      }
   }

   /** allow thread a smooth stop by stopping thread run loop.
    */
   public void requestStop() {
      stop = true;
   }
}
