/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.
 * 31 January 2006
 * Mike Muir
 *******************************************************************************
 * @author mikem
 * version 1.4.2 - 31 January 2006 - Creation
 * @version 1.4.4 6-May-2006 Fixed javaDoc and coding standards
 */

package com.nemmco.participantbatcher.util;

import java.io.File;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;

import org.apache.log4j.Logger;

/** Runs automatically when PB exits to write the HttpDataStore to file.
 *
 */
public class HttpShutdownHook extends Thread {

   private static Logger logger = Logger.getLogger(HttpShutdownHook.class);

   /** Creates a new instance of HttpShutdownHook */
   public HttpShutdownHook() {}

   private String getFileName() {
      return Constants.getConstant().getMonitorFileDir() + "/" + Constants.getConstant().getInstanceName() + ".ser";
   }

   /** will attempt to load the ".ser" file and if it fails return with
    * the HttpDataStore object as null
    */
   public HttpDataStore loadFile() {

      HttpDataStore hds = null;
      try {

         // get it from the monitor file location
         File f = new File(this.getFileName());
         Util_Logger.debug(logger, "Loading file " + this.getFileName());

         // write the HttpDataStore to file
         ObjectInputStream in = new ObjectInputStream(new FileInputStream(f));
         hds = (HttpDataStore)in.readObject();
         in.close();
         Util_Logger.debug(logger, "Read the serializer file");

      } catch (FileNotFoundException e) {
         Util_Logger.debug(logger, "Failed to find the serialization file");
      } catch (Exception e) {
         Util_Logger.warning(logger, "Failed to read the serializer file", e);
      }
      return hds;

   }

   /** will save the HttpDataStore as a ".ser" file on shutdown
    */
   public void run() {

      try {

         // put in same place as the monitor file
         HttpDataStore hds;
         hds = HttpDataStore.getHttpDataStore();
         File f = new File(this.getFileName());
         System.out.print("Writing serializer file");

         // write the HttpDataStore to file
         ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(f));
         out.writeObject(hds);
         out.close();
         Util_Logger.debug(logger, "Wrote the serializer file");
         System.out.println("  ... successful");

      } catch (Exception e) {
         Util_Logger.warning(logger, "Failed to write the serializer file", e);
      }
   }

}
