/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 */

package com.nemmco.participantbatcher.xml;

/**
 * This class is used to hold infomation from original XML file, which can
 * be used to generate acknowledgement (ack) file.
 *
 * author Kevin Jin
 * version 1 (19/09/2003) -- Creation
 * version 1.4.4 6-May-2006 Fixed javaDoc and coding standards
 * version 1.5.3 - 10-Jul-2009 5796 Add schemaVersion.
 * @author Deepak Gupta
 * @version 46.93 - 6-Mar-2018 - DFCT0022009
 */
public class ZipFileInfo {

   // Fields
   private String fromName = "";
   private String fromDesc = "";
   private String toName = "";
   private String toDesc = "";
   private String initMsgId = "";
   private String transGroup = "";
   private String priority = "";
   private String securityContext = "";
   private String errorMessage = "";
   private String schemaVersion = "";

   private boolean valid = false;
   private boolean processSuccess = true;
	private boolean fileEligibleToBeProcessed = true;

   /**
    * Constructor
    */
   public ZipFileInfo() {}

   // Getters and Setters

   /**
    * Access method to set original XML message from user name.
    *
    * @param param string
    */
   public void setFromName(String param) {
      fromName = param;
   }

   /**
    * Access method to get original XML message from user name.
    *
    * @return string
    */
   public String getFromName() {
      return fromName;
   }

   /**
    * Access method to set original XML message from user description.
    *
    * @param param string
    */
   public void setFromDesc(String param) {
      fromDesc = param;
   }

   /**
    * Access method to get original XML message from user description.
    *
    * @return string
    */
   public String getFromDesc() {
      return fromDesc;
   }

   /**
    * Access method to set original XML message to user name.
    *
    * @param param string
    */
   public void setToName(String param) {
      toName = param;
   }

   /**
    * Access method to get original XML message to user name.
    *
    * @return string
    */
   public String getToName() {
      return toName;
   }

   /**
    * Access method to set original XML message to user description.
    *
    * @param param string
    */
   public void setToDesc(String param) {
      toDesc = param;
   }

   /**
    * Access method to get original XML message to user description.
    *
    * @return string
    */
   public String getToDesc() {
      return toDesc;
   }

   /**
    * Access method to set original XML message ID.
    *
    * @param param string
    */
   public void setInitMsgId(String param) {
      initMsgId = param;
   }

   /**
    * Access method to get original XML message ID.
    *
    * @return string
    */
   public String getInitMsgId() {
      return initMsgId;
   }

   /**
    * Access method to set original XML message transaction group.
    *
    * @param param string
    */
   public void setTransGroup(String param) {
      transGroup = param;
   }

   /**
    * Access method to get original XML message transaction group.
    *
    * @return string
    */
   public String getTransGroup() {
      return transGroup;
   }

   /**
    * Access method to set original XML message priority.
    *
    * @param param string
    */
   public void setPriority(String param) {
      priority = param;
   }

   /**
    * Access method to get original XML message priority.
    *
    * @return string
    */
   public String getPriority() {
      return priority;
   }

   /**
    * Access method to set original XML message security context.
    *
    * @param param string
    */
   public void setSecurityContext(String param) {
      securityContext = param;
   }

   /**
    * Access method to get original XML message security context.
    *
    * @return string
    */
   public String getSecurityContext() {
      return securityContext;
   }

   /**
    * Access method to set original XML file validation error message.
    *
    * @param param string
    */
   public void setErrorMessage(String param) {
      errorMessage = param;
   }

   /**
    * Access method to get original XML file validation error message.
    *
    * @return string
    */
   public String getErrorMessage() {
      return errorMessage;
   }

   /**
    * Access method to set schema version.
    *
    * @param param string
    */
   public void setSchemaVersion(String param) {
      schemaVersion = param;
   }

   /**
    * Access method to get schema version.
    *
    * @return string
    */
   public String getSchemaVersion() {
      return schemaVersion;
   }

   /**
    * Access method to set original XML file validility.
    *
    * @param param string
    */
   public void setIsValid(boolean param) {
      valid = param;
   }

   /**
    * Access method to get original XML file validility.
    *
    * @return string
    */
   public boolean isValid() {
      return valid;
   }

   /**
    * Access method to set original XML file validation process status.
    *
    * @param param boolean
    */
   public void setProcessSuccess(boolean param) {
      processSuccess = param;
   }

   /**
    * Access method to get original XML file validation process status.
    *
    * @return string
    */
   public boolean isProcessSuccess() {
      return processSuccess;
   }

	/**
	 * Access method to set original XML file processable status.
	 *
	 * @param param boolean
	 */
	public void setFileEligibleToBeProcessed(boolean param) {
		fileEligibleToBeProcessed = param;
	}

	/**
	 * Access method to get original XML file processable status.
	 *
	 * @return string
	 */
	public boolean isFileEligibleToBeProcessed() {
		return fileEligibleToBeProcessed;
	}

}

